import app/web
import gleam/bit_array
import gleam/bytes_tree
import gleam/crypto
import gleam/dynamic/decode
import gleam/erlang/process
import gleam/float
import gleam/http/request.{type Request}
import gleam/http/response.{type Response}
import gleam/io
import gleam/json
import gleam/list
import gleam/option.{None, Some}
import gleam/otp/actor
import gleam/result
import gleam/string
import mist.{type Connection, type ResponseData}

pub fn handle_request(
  req: Request(Connection),
  ctx: web.Context,
) -> Response(ResponseData) {
  io.println("Got request: " <> req.path)
  case request.path_segments(req) {
    [] -> serve_wasm(ctx, "index.html", "text/html")
    ["static", "frontend.js"] ->
      serve_wasm(ctx, "frontend.js", "text/javascript")
    ["static", "frontend_bg.wasm"] ->
      serve_wasm(ctx, "frontend_bg.wasm", "application/wasm")
    ["ws"] -> serve_websocket(ctx, req)
    _ -> response.set_body(response.new(404), mist.Bytes(bytes_tree.new()))
  }
}

fn serve_wasm(
  ctx: web.Context,
  file: String,
  mimetype: String,
) -> Response(ResponseData) {
  let wasm_path = string.join([ctx.config.static_path, file], "/")
  mist.send_file(wasm_path, offset: 0, limit: None)
  |> result.map(fn(file) {
    response.new(200)
    |> response.prepend_header("content-type", mimetype)
    |> response.set_body(file)
  })
  |> result.lazy_unwrap(fn() {
    response.new(404)
    |> response.set_body(mist.Bytes(bytes_tree.new()))
  })
}

fn serve_websocket(
  ctx: web.Context,
  req: Request(Connection),
) -> Response(ResponseData) {
  mist.websocket(
    request: req,
    on_init: fn(_conn) { #(ctx, Some(ctx.selector)) },
    on_close: fn(_state) { io.println("goodbye!") },
    handler: handle_ws_message,
  )
}

fn decode_error_to_string(decode_err: decode.DecodeError) -> String {
  "Path: "
  <> string.join(decode_err.path, ".")
  <> " Expected: "
  <> decode_err.expected
  <> " Got: "
  <> decode_err.found
}

fn decode_errors_to_string(decode_errs: List(decode.DecodeError)) -> String {
  string.join(list.map(decode_errs, decode_error_to_string), "\n")
}

fn handle_ws_message(state, conn, message) {
  io.println("Got WS message")
  case message {
    mist.Text("ping") -> {
      io.println("Got Ping")
      let assert Ok(_) = mist.send_text_frame(conn, "pong")
      actor.continue(state)
    }
    mist.Text(msg) -> {
      echo msg
      case json.parse(msg, web.client_request_decoder()) {
        Error(err) -> {
          let err_msg = case err {
            json.UnableToDecode(decode_err) ->
              "Decode Error: " <> decode_errors_to_string(decode_err)
            json.UnexpectedByte(bad_byte) -> "Unexpeted byte: " <> bad_byte
            json.UnexpectedEndOfInput -> "Reached EoF"
            json.UnexpectedFormat(_) -> "Unexpected format"
            json.UnexpectedSequence(seq_msg) ->
              "Unexpected Sequence: " <> seq_msg
          }
          io.println("Could not parse client message: " <> err_msg)
        }
        Ok(req) -> {
          let resp = handle_client_msg(state, req)
          echo resp
          let resp_str =
            web.encode_game_response(resp)
            |> json.to_string

          let assert Ok(_) = mist.send_text_frame(conn, resp_str)
          Nil
        }
      }

      actor.continue(state)
    }
    mist.Binary(_) -> {
      actor.continue(state)
    }
    mist.Custom(_) -> {
      actor.continue(state)
    }
    mist.Closed | mist.Shutdown -> actor.Stop(process.Normal)
  }
}

fn handle_client_msg(
  _ctx: web.Context,
  req: web.ClientRequest,
) -> web.GameResponse {
  case req.msg {
    web.BuzzIn(time) -> {
      io.println("Got buzz in @ " <> float.to_string(time))
      web.AckBuzzer
    }
    web.Register(username, gamecode) -> {
      let secret = crypto.strong_random_bytes(16)
      let token =
        crypto.new_hasher(crypto.Sha512)
        |> crypto.hash_chunk(secret)
        |> crypto.hash_chunk(bit_array.from_string(username))
        |> crypto.hash_chunk(bit_array.from_string(gamecode))
        |> crypto.digest

      let token_hash =
        crypto.new_hasher(crypto.Sha512)
        |> crypto.hash_chunk(token)
        |> crypto.digest

      io.println("New token: " <> bit_array.base64_encode(token_hash, True))

      let token = bit_array.base64_encode(token, True)

      web.JoinResponse(username, token)
    }
  }
}
