use web_time::{SystemTime, UNIX_EPOCH};

use crate::StateTransition;
use crate::context::Context;
use crate::model::ClientMessage;
use crate::model::buzz_in::BuzzIn;
use crate::screen::Screen;
use macroquad::prelude::*;

#[derive(Default)]
pub struct PlayerScreen {
    score: i32,
    button_pressed: bool,
}

impl Screen for PlayerScreen {
    fn handle_messages(&mut self, ctx: &mut Context) -> Option<StateTransition> {
        let msg = ctx.recv_msg();

        if let Some(msg) = msg {
            match msg {
                crate::model::GameResponse::JoinResponse { .. } => {}
                crate::model::GameResponse::AckBuzzer => {
                    info!("Button pressed ack");
                    self.button_pressed = true;
                }
                crate::model::GameResponse::ResetBuzzer => {
                    info!("Button press reset");
                    self.button_pressed = false;
                }
                crate::model::GameResponse::UpdatePountTotal { score } => {
                    info!("Score updated to: {}", score);
                    self.score = score;
                }
            }
        }

        None
    }

    fn handle_frame(&mut self, ctx: &mut Context) {
        clear_background(BEIGE);

        let button_color = if self.button_pressed {
            Color::from_hex(0xaa0000)
        } else {
            RED
        };

        draw_text(
            &format!("Score: {}", self.score),
            screen_width() * 0.5,
            screen_height() * 0.25,
            50.0,
            BLACK,
        );

        let button_size = (screen_width() * 0.33).min(screen_height() * 0.20);
        draw_circle(
            screen_width() * 0.5,
            screen_height() * 0.5,
            button_size * 1.1,
            DARKBROWN,
        );
        draw_circle(
            screen_width() * 0.5,
            screen_height() * 0.5,
            button_size,
            button_color,
        );

        if !self.button_pressed && is_mouse_button_pressed(MouseButton::Left) {
            let loc = Vec2::from(mouse_position());
            let normalize = loc - Vec2::new(screen_width() * 0.5, screen_height() * 0.5);

            if normalize.length() < button_size {
                let timestamp = SystemTime::now();
                let unix_timestamp = timestamp.duration_since(UNIX_EPOCH).expect("Time invalid");

                let time = unix_timestamp.as_secs_f64();

                info!("Button pressed @ {}", time);
                let buzz_in = BuzzIn { time };

                ctx.send_msg(&ClientMessage::BuzzIn(buzz_in));
            }
        }
    }
}
